# !SetUpBookStack.py (Version 1.0)
# Script for setting the limits, initial values, visibility, etc for parameters in all the books in a bookstack figure (or stack of such)

import poser,random

# I put my scripts into a class simply to avoid possible conflict with stuff in the __main__ module.
# From Sep 2024 I've added the 'main' and 'del' stuff at the bottom as per D3Ds 'UnHide Dials' script to tidy things up after running the script
class cheapskatesTemporaryClass(object):
	
	# List of all the BookStack4 figures linked underneath the selected one
	figlist = []
	
	# Recursive function to get any BookStack4 figures to which this, or any of its parent, BookStack4 figures are parented
	def getAnyParentStackFigures(self,thisfig):
		act = thisfig.ParentActor()
		parentact = act.Parent()
		parentactname = parentact.Name()
		if parentactname[:10] == "TopOfStack":
			if parentact.ItsFigure():
				parentfig = parentact.ItsFigure()
				figname = parentfig.Name()
				if figname[:9] == "BookStack":
					self.figlist.append(figname)
					self.getAnyParentStackFigures(parentfig)
		return
	
	# Recursive function to get any BookStack4 figures that are parented to TopOfStack of the main, or of any nested, BookStack4 figure
	def getAnyChildStackFigures(self, thisfig):
		act = thisfig.ActorByInternalName("top_of_stack")
		actlist = act.Children()
		for a in actlist:
			if a.ItsFigure():
				childfig = a.ItsFigure()
				figname = childfig.Name()
				if figname[:9] == "BookStack":
					self.figlist.append(figname)
					self.getAnyChildStackFigures(childfig)
					
		return
	
	# Print a parameter's data
	def printParameter(self,p):
		print("          - "+p.Name()+", "+str(p.Value())+", "+str(p.InitValue())+", "+str(p.MinValue())+", "+str(p.MaxValue())+", "+str(p.Hidden())+", "+str(p.ForceLimits()))
		return
	
	# Set up a parameter
	def setupParameter(self,p,val,init,min,max,hid,frclim):
		#self.printParameter(p)
		p.SetValue(val)
		p.SetInitValue(init)
		p.SetValueFrame(init,0)
		p.SetMinValue(min)
		p.SetMaxValue(max)
		p.SetHidden(hid)
		p.SetForceLimits(frclim)
		return
	
	def main(self):
		
		# Get Poser version for doing version-specific stuff
		poserversion = '0'
		posermainversion = '0'
		try:
			self.poserversion = poser.AppVersion()
			temp = self.poserversion.split('.')
			self.posermainversion = temp[0]
		except:
			pass
			
		# Abort if the user gets cold feet right at the start
		confirmed = poser.DialogSimple.YesNo("FOR SETTING UP DEFAULT PARAMETER VISIBLITY AND LIMITS\n\nDo you wish to continue?")
		if not confirmed:
			return
			
		# Abort if the selected figure's not a BookStack4
		fig = poser.Scene().CurrentFigure()
		if not (fig.Name()[:9] == "BookStack"):
			poser.DialogSimple.MessageBox("This script is only for use with the 3DCheapskate's 'BookStack' figures.\n\nScript aborted, no changes made.")
			return
		
		# Get parent stacks recursively, and then reverse the list so that the ultimate parent of the whole stack is first
		parentstoo=poser.DialogSimple.YesNo("Parent bookstack figures too?")
		if parentstoo:
			self.getAnyParentStackFigures(fig)
			self.figlist.reverse()
		
		# ...then add the current figure to the list
		self.figlist.append(fig.Name())
		
		# ...then get child stacks recursively
		kidstoo=poser.DialogSimple.YesNo("Child bookstack figures too?")
		if kidstoo:
			self.getAnyChildStackFigures(fig)
				
		bkthisfig = 0 # number of books in this figure (obtained from figure name)
		for f in self.figlist:
		
			# Get the figure
			fig = poser.Scene().Figure(f)
			
			# Print figure and actor names
			print ("\nList of actors in figure: : " + fig.Name())
			actlist = fig.Actors()
			for a in actlist:
				print ("  - " + a.Name() + " (" + a.InternalName() + ")")
			
			# Extract the number of books in the figure from the name, e.g. "BookStack4 - books 1-4", "BookStack8 - books 1-8", "BookStack32 - books 1-32"
			# (This is in case I ever create the larger BookStack figures)
			if fig.Name()[9].isdigit():
				if fig.Name()[10].isdigit():
					bkthisfig = int(fig.Name()[9])*10 + int(fig.Name()[10])
				else:
					bkthisfig = int(fig.Name()[9])
		
			print("\n")		
			# For each of the books in the figure
			i = 1
			while i < (bkthisfig+1):
				
				# Get the relevant bone/bodypart
				bone = "book_" + str(i)
				act = fig.ActorByInternalName(bone)
				print ("\n   Parameter settings for '"+ act.Name() + "' (" + act.InternalName() + ") of '"+fig.Name()+"'")
				print("          - Name, Value, InitValue, MinValue, MaxValue, Hidden, ForceLimits")
								
				# Set up geometry variant
				pname = "Book" + str(i)+"Geom"
				p = act.Parameter(pname)
				
				# Set up morphs + adjustments
				self.setupParameter(act.Parameter("Splay"),0,0,-1,1,0,1)					
				self.setupParameter(act.Parameter("Oblique"),0,0,-1,1,0,1)
				self.setupParameter(act.Parameter("Shear"),0,0,-1,1,0,1)
				self.setupParameter(act.Parameter("Plump"),0,0,-1,1,0,1)
				self.setupParameter(act.Parameter("Shear"),0,0,-1,1,0,1)
				self.setupParameter(act.Parameter("Twist"),0,0,-1,1,0,1)
				self.setupParameter(act.Parameter("Curl (Paperback only)"),0,0,-1,1,0,1)
				
				# Set up rotation
				self.setupParameter(act.Parameter("Rotate"),0,0,-360,360,0,1)
				
				# Set up position
				self.setupParameter(act.Parameter("Nudge sideways"),0,0,-1,1,0,1)
				self.setupParameter(act.Parameter("Nudge lengthwise"),0,0,-1,1,0,1)
				
				# Set up dimensions
				self.setupParameter(act.Parameter("Cover proportions"),1,1,0.9,1.05,0,1)
				self.setupParameter(act.Parameter("Book size"),1,1,0.95,1.125,0,1)
				self.setupParameter(act.Parameter("Book thickness"),1,1,0.875,1.125,0,1)
				
				# Set up MTs
				self.setupParameter(act.Parameter("MT HBc"+str(i)+" Plump"),0,0,-1,1,1,1)
				self.setupParameter(act.Parameter("MT HBc"+str(i)+" Shear"),0,0,-1,1,1,1)
				self.setupParameter(act.Parameter("MT HBc"+str(i)+" Twist"),0,0,-1,1,1,1)
				self.setupParameter(act.Parameter("MT HBf"+str(i)+" Plump"),0,0,-1,1,1,1)
				self.setupParameter(act.Parameter("MT HBf"+str(i)+" Shear"),0,0,-1,1,1,1)
				self.setupParameter(act.Parameter("MT HBf"+str(i)+" Twist"),0,0,-1,1,1,1)
				self.setupParameter(act.Parameter("MT PB"+str(i)+" Curl"),0,0,-1,1,1,1)
				self.setupParameter(act.Parameter("MT PB"+str(i)+" Plump"),0,0,-1,1,1,1)
				self.setupParameter(act.Parameter("MT PB"+str(i)+" Shear"),0,0,-1,1,1,1)
				self.setupParameter(act.Parameter("MT PB"+str(i)+" Twist"),0,0,-1,1,1,1)
				
				# Set up standard transforms
				self.setupParameter(act.Parameter("xRotate"),0,0,-5,5,1,1)
				self.setupParameter(act.Parameter("yRotate"),0,0,-100000,100000,1,0)
				self.setupParameter(act.Parameter("zRotate"),0,0,-5,5,1,1)
				self.setupParameter(act.Parameter("Scale"),1,1,-100000,100000,1,0)
				self.setupParameter(act.Parameter("xScale"),1,1,-100000,100000,1,0)
				self.setupParameter(act.Parameter("yScale"),1,1,-100000,100000,1,0)
				self.setupParameter(act.Parameter("zScale"),1,1,-100000,100000,1,0)
				self.setupParameter(act.Parameter("xTran"),0,0,-100000,100000,1,0)
				self.setupParameter(act.Parameter("yTran"),0,0,-0.0024,0.01,1,1)
				self.setupParameter(act.Parameter("zTran"),0,0,-100000,100000,1,0)
				
				# Increment book number for next iteration
				i = i + 1
			
		# Redraw what we've modified	
		poser.Scene().Draw()


def main():
	poser.DialogSimple.MessageBox("Running 3DCheapskates '!SetUpBookStack.py' script\n\n(N.B. You should see a similar message when the script finishes. If you don't then the script has crashed!)")
	cheapskatesTemporaryInstance = cheapskatesTemporaryClass()
	cheapskatesTemporaryInstance.main()
	poser.DialogSimple.MessageBox("3DCheapskates '!SetUpBookStack.py' script all done.")

main()

del main
del cheapskatesTemporaryClass